include "FixedCode.inc"
include "VVram.inc"

Vram equ 08000h

VramWidth equ 80
VramHeight equ 25
CharHeight equ 8
VramRowSize equ VramWidth*CharHeight
VramTop equ Vram

PatternSize equ CharHeight*2

PatternRam equ 4000h
Port31Save equ 0e6c6h

ext VVramFront_, RowFlags_

dseg
Backup:
    defs VVramWidth*VVramHeight

cseg
ToRam:
    push af
        ld a,(Port31Save)
        or 02h
        out (31h),a
        ld (Port31Save),a
    pop af
ret

ToRom:
    push af
        ld a,(Port31Save)
        and not 02h
        out (31h),a
        ld (Port31Save),a
    pop af
ret

PatternAddress:
    push de
        ld l,a
        ld h,0
        add hl,hl ;*2
        add hl,hl ;*4
        add hl,hl ;*8
        add hl,hl ;*16
        ld de,PatternRam
        add hl,de
    pop de
ret


; void MakePatternMono(byte c, ptr<byte> pSource, byte count, byte color);
dseg
    MakePatternMono_@Param3: public MakePatternMono_@Param3
    defb 0
cseg
MakePatternMono_: public MakePatternMono_
    push af | push hl | push de | push bc
        di
            call ToRam
            exx
                call PatternAddress
            exx
            do
                ld a,(MakePatternMono_@Param3) ; color
                and 3
                exx
                    ld d,a
                exx
                ld l,8
                do
                    ld a,(de)
                    inc de
                    exx
                        ld c,a
                    exx
                    ld h,2
                    do
                        exx
                            ld e,0
                        exx
                        ld b,4
                        do
                            exx
                                sla e
                                sla e
                                sla c
                                if c
                                    ld a,e
                                    or d
                                    ld e,a
                                endif
                            exx
                        dwnz
                        exx
                            ld (hl),e
                            inc hl
                        exx
                        dec h
                    while nz | wend
                    dec l
                while nz | wend
                dec c
            while nz | wend
            call ToRom
        ei
    pop bc | pop de | pop hl | pop af
ret


; void MakePatternColor(byte c, ptr<byte> pSource, byte count);
MakePatternColor_: public MakePatternColor_
    push af | push hl | push de | push bc
        di
            call ToRam
            exx
                call PatternAddress
            exx
            do
                ld b,16
                do
                    ld a,(de)
                    inc de
                    exx
                        ld (hl),a
                        inc hl
                    exx
                dwnz
                dec c
            while nz | wend
            call ToRom
        ei
    pop bc | pop de | pop hl | pop af
ret


ClearRowFlags:
    xor a
    ld hl,RowFlags_
    ld b,4
    do
        ld (hl),a
        inc hl
    dwnz
ret


; void ClearScreen();
ClearScreen_:   public ClearScreen_
    push af | push hl | push de | push bc
        call ClearVram

        ld hl,Backup
        ld bc,VVramWidth * VVramHeight
        do
            ld (hl),0
            inc hl
            dec bc
            ld a,c
            or b
        while nz | wend
        call ClearRowFlags
    pop bc | pop de | pop hl | pop af
ret


; word Put(word vram, byte c);
Put_: public Put_
    push af | push hl | push de | push bc
        ld a,e
        ex de,hl
            call PatternAddress
        ex de,hl
        di
            call ToRam
            call WriteVram
            call ToRom
        ei
    pop bc | pop de | pop hl | pop af
    inc hl | inc hl
ret


VVramToVramRow:
    ld b,VVramWidth
    do 
        push bc 
            ld a,(de)
            inc de
            cp (hl)
            if nz
                ld (hl),a
                exx
                    push de
                        ld e,a
                        call Put_
                    pop de
                exx
            else
                exx
                    inc hl
                    inc hl
                exx
            endif
            inc hl
        pop bc
    dwnz
    exx
        push bc
            ld bc,VramRowSize-VVramWidth*2
            add hl,bc
        pop bc
    exx
ret


; void VVramToVramAll();
VVramToVramAll_: public VVramToVramAll_
    push af | push hl | push de | push bc
        ld de,VVramFront_
        ld hl,Backup
        exx
            ld hl,Vram
        exx
        ld c, VVramHeight
        do
            call VVramToVramRow
            dec c
        while nz | wend
        call ClearRowFlags
    pop bc | pop de | pop hl | pop af
ret


; void VVramToVramChanged();
VVramToVramChanged_: public VVramToVramChanged_
    push af | push hl | push de | push bc
        ld de,VVramFront_
        ld hl,Backup
        exx
            ld hl,Vram
            ld de,RowFlags_
            ex de,hl
                ld c,1
                ld b,(hl)
                inc hl
            ex de,hl
        exx
        ld c, VVramHeight ; vertical count    
        do
            exx
                ld a,b
                and c
            exx
            if nz
                call VVramToVramRow            
            else
                push bc
                    ld bc,VVramWidth
                    add hl,bc
                    ex de,hl
                        add hl,bc
                    ex de,hl
                pop bc
                exx
                    push bc
                        ld bc,VramRowSize
                        add hl,bc
                    pop bc
                exx
            endif
            exx
                sla c
                if z
                    ex de,hl
                        ld c,1
                        ld b,(hl)
                        inc hl
                    ex de,hl
                endif
            exx
            dec c
        while nz | wend
        call ClearRowFlags
    pop bc | pop de | pop hl | pop af
ret
